#!/usr/bin/perl -w

BEGIN {
    die "GSDLHOME not set\n" unless defined $ENV{'GSDLHOME'};
    die "GSDLOS not set\n" unless defined $ENV{'GSDLOS'};
    unshift (@INC, "$ENV{'GSDLHOME'}/perllib");
    unshift (@INC, "$ENV{'GSDL3HOME'}/lib/perl/cpan");
}

use colcfg;
use util;
use parsargv;
use FileHandle;
use XML::Writer;

&main();
sub print_usage() {
    print STDOUT "Usage: convert_coll_from_gs2.pl [options] coll-name\n";
    print STDOUT "options:\n";
    
    print STDOUT "   -collectdir         Directory where collection lives.\n";
    print STDOUT "   -verbosity          Controls the amount of output.\n\n"; 
}

sub main {

    my $defaultlang = 'en';
    my ($collectdir);
    # note that no defaults are passed for most options as they're set
    # later (after we check the collect.cfg file)
    if (!parsargv::parse(\@ARGV,
			 'verbosity/\d+/', \$verbosity,
			 'collectdir/.*/', \$collectdir)) {
	&print_usage();
	die "\n";
    }

    # get and check the collection name
    if (($collection = &util::use_collection(@ARGV, $collectdir)) eq "") {
	&print_usage();
	die "\n";
    }
    
    $collconfigfilename = &util::filename_cat ($ENV{'GSDLCOLLECTDIR'}, "etc", "collect.cfg");
    my $collectcfg;
    if (-e $collconfigfilename) {
	$collectcfg = &colcfg::read_collect_cfg ($collconfigfilename);

    } else {
	print STDERR "collect.cfg not found!!";
	die "\n";
    }
    

    $buildconfigfilename = &util::filename_cat ($ENV{'GSDLCOLLECTDIR'}, "index", "build.cfg");
    my $buildcfg;
    if (-e $buildconfigfilename) {
	$buildcfg = &colcfg::read_build_cfg ($buildconfigfilename);

    } else {
	print STDERR "build.cfg not found!!";
	die "\n";
    }
    

     
    my $colloutfile = &util::filename_cat ($ENV{'GSDLCOLLECTDIR'}, "etc", "collectionConfig.xml");
    if (-e $colloutfile) {
	print STDOUT "collectionConfig file already exists! overwriting it!\n";
	
    }
   
    my $buildoutfile = &util::filename_cat ($ENV{'GSDLCOLLECTDIR'}, "index", "buildConfig.xml");
    if (-e $buildoutfile) {
	print STDOUT "buildConfig file already exists! overwriting it!\n";
	
    }

    my $buildtype;
    if (defined $collectcfg->{'buildtype'}) {
	$buildtype = $collectcfg->{'buildtype'};
    } else {
	$buildtype = 'mg';
    }
    
    my $buildoutput = new IO::File(">$buildoutfile");
    my $buildwriter = new XML::Writer(OUTPUT => $buildoutput, NEWLINES => 1);
    
    $buildwriter->startTag('buildConfig', 'xmlns:gsf'=>"http://www.greenstone.org/configformat");
    
    my $colloutput = new IO::File(">$colloutfile");
    my $collwriter = new XML::Writer(OUTPUT => $colloutput, NEWLINES => 1);
    
    $collwriter->startTag('collectionConfig', 'xmlns:gsf'=>"http://www.greenstone.org/configformat");
    
    #output the collection metadata to the collectionConfig file
    my $collectionmeta = $collectcfg->{'collectionmeta'};
    $collwriter->startTag('metadataList');
    
    %name_map = (
		 'collectionname', 'colName',
		 'collectionextra', 'colDescription',
		 'iconcollection', 'colIcon',
		 'iconcollectionsmall', 'colIconSmall');
    
    if (defined $collectionmeta) {
	foreach $entry ( keys %$collectionmeta) {
	    if (defined $name_map{$entry}) {
		$name= $name_map{$entry};
		foreach $lang (keys %{$collectionmeta->{$entry}}) {
		    $value = $collectionmeta->{$entry}->{$lang};
		    if ($entry =~ /^icon/) {
			$value = format_icon_value($value);
		    }
		    outputmetadata($collwriter, $name, $lang, $value);
		}
	    }
	}
    } 
    $collwriter->endTag('metadataList');
    
   # output building metadata to build config file 
    my $numdocs = $buildcfg->{'numdocs'};
    $buildwriter->startTag('metadataList');
    $buildwriter->startTag('metadata',  'name'=>'numDocs');
    $buildwriter->characters($numdocs);
    $buildwriter->endTag('metadata');
    $buildwriter->endTag('metadataList');
    
    
    #indexes
    my $indexmap = {};
    if (defined $buildcfg->{'indexmap'}) {
	$indexmap_t = $buildcfg->{'indexmap'};
	foreach $i (@$indexmap_t) {
	    my ($k, $v) = $i =~ /^(.*)\-\>(.*)$/;
	    $indexmap->{$k} = $v;
	}
    } else {
	print STDERR "indexmap not defined";
    }
    my $defaultindex;
    if (defined $collectcfg->{'defaultindex'}) {
	$defaultindex = $collectcfg->{'defaultindex'};
    } else {
	print STDERR "defaultindex not defined";
    }
    $defaultindex = $indexmap->{$defaultindex};
    
    # levels for index
    my $level_string = "Document";
    if (defined $collectcfg->{'levels'}) {
	foreach $l (@{$collectcfg->{'levels'}}) {
	    $level_string .=",$l";
	}
    }
    # format stuff
    my $format = $collectcfg->{'format'};

    #output the search stuff to coll cfg
    $collwriter->startTag('search', 'type'=>$buildtype);
    foreach $i (keys %$indexmap) {
	$shortname = $indexmap->{$i};
	if ($buildtype eq 'mgpp') {
	    $collwriter->startTag('index', 'name'=>$shortname, 'content'=>$i, 
				  'level'=>$level_string);
	} elsif ($buildtype eq 'mg') {
	    ($level, $content) = $i =~ /^(.*):(.*)$/;
	    $collwriter->startTag('index', 'name'=>$shortname, 'content'=>$content, 'level'=>$level);
	}
	#find the coll meta stuff
	$indexdisplay = ".$i";
	foreach $lang (keys %{$collectionmeta->{$indexdisplay}}) {
	    $value = $collectionmeta->{$indexdisplay}->{$lang};
	    output_display($collwriter, $lang, $value);
	}
	$collwriter->endTag('index');
	
    }

    # add in the format stuff
    if (defined $format->{'SearchVList'}) {

	$collwriter->startTag('format');
	write_format($collwriter, $format->{'SearchVList'});
	$collwriter->endTag('format');
    }
    
    $collwriter->endTag('search');
    
    $buildwriter->startTag('serviceRackList');
    
    my @levels = ('Document');
    my $defaultlevel = 'Document';

    #do the retrieve service
    # assume mgpp or mg 
    if ($buildtype eq 'mgpp') {
	#for each level
	if (defined $collectcfg->{'levels'}) {
	    push @levels, @{$collectcfg->{'levels'}};
	    
	    foreach $l(@levels){
		$defaultlevel = 'Section' if $l eq 'Section';
	    }
	}

	$buildwriter->startTag('serviceRack', 'name'=>'GS2MGPPRetrieve');
	$buildwriter->emptyTag('defaultLevel', 'name'=>$defaultlevel);
	$buildwriter->startTag('levelList');
	foreach $level (@levels) {
	    $buildwriter->emptyTag('level', 'name'=>$level);
	}
	$buildwriter->endTag('levelList');
	
	
    } else {
	$buildwriter->startTag('serviceRack', 'name'=>'GS2MGRetrieve');
	$buildwriter->emptyTag('defaultIndex', 'name'=>$defaultindex);
	
    }

    # add in the classifiers if needed

    my $count = 1;
    my $phind = 0;
    my $started_classifiers = 0;
    if (defined $collectcfg->{'classify'}) { 
	$collwriter->startTag('browse');
	my $classifiers = $collectcfg->{'classify'};
	foreach $cl (@$classifiers) {
	    $name = "CL$count";
	    $count++;
	    my ($classname) = @$cl[0];
	    if ($classname =~ /^phind$/i) {
		$phind=1;
		#should add it into coll config classifiers
		next;
	    }
	    
	    my $document_interleave = "true";
	    my $orientation = "vertical";
	    my $nodelinks = 0;
	    if ($classname eq "AZList" || $classname eq "AZCompactList") { #there may be others
		$document_interleave = "false";
		$orientation = "horizontal";
		$nodelinks = 1;
	    }
	    if (not $started_classifiers) {
		$buildwriter->startTag('classifierList');
		$started_classifiers = 1;
	    }
	    my $content = ''; #use buttonnmae first, then metadata
	    if ($classname eq "DateList") {
		$content = "Date";
	    } else {
		for ($i=0; $i<scalar(@$cl); $i++) {
		    $arg = @$cl[$i];
		    if ($arg eq "-buttonname"){
			$content = @$cl[$i+1];
			last;
		    }elsif ($arg eq "-metadata") {
			$content = @$cl[$i+1];
		    }
		    
		}
	    }
	    
	    $buildwriter->emptyTag('classifier', 'name'=>$name, 'content'=>$content, 'orientation'=>$orientation, 'documentInterleave'=>$document_interleave);
	    
	    $collwriter->startTag('classifier', 'name'=>$name, 'content'=>$content, 'type'=>$classname, 'level'=>'Document');
	    $vlist = $name."VList";
	    $hlist = $name."HList";
	    $dlist = "";
	    if ($classname eq "DateList") {
		$dlist = "DateList";
	    }
	    if ($nodelinks || defined $format->{$vlist} || defined $format->{$hlist} || defined $format->{$dlist}) {
		$collwriter->startTag('format');
		if ($nodelinks) {
		    write_nodelinks_format($collwriter);
		}
		
		if (defined $format->{$vlist}) {
		    write_format($collwriter, $format->{$vlist});
		}
		if (defined $format->{$hlist}) {
		    write_format($collwriter, $format->{$hlist});
		}
		if (defined $format->{$dlist}) {
		    write_format($collwriter, $format->{$dlist});
		}
		$collwriter->endTag('format');
	    }
	    $collwriter->endTag('classifier');
	    } #foreach classifier
	if ($started_classifiers) {
	    # end the classifiers
	    $buildwriter->endTag('classifierList');
	}
	
	$collwriter->endTag('browse');
    }
    # close off the Retrieve service
    $buildwriter->endTag('serviceRack');
    
    # the phind classifier is a separate service
    if ($phind) {
	# if phind classifier
	$buildwriter->emptyTag('serviceRack', 'name'=>'PhindPhraseBrowse');
    }
   
    # do the search service
    if ($buildtype eq 'mgpp') {
		
	$buildwriter->startTag('serviceRack', 'name'=>'GS2MGPPSearch');
	$buildwriter->emptyTag('defaultLevel', 'name'=>$defaultlevel);
	$buildwriter->startTag('levelList');
	foreach $level (@levels) {
	    $buildwriter->emptyTag('level', 'name'=>$level);
	}
	$buildwriter->endTag('levelList');
	
	#fieldlist
	my $fieldmap = {};
	if (defined $buildcfg->{'indexfieldmap'}) {
	    $fieldmap_t = $buildcfg->{'indexfieldmap'};
	    foreach $f (@$fieldmap_t) {
		my ($k, $v) = $f =~ /^(.*)\-\>(.*)$/;
		$fieldmap->{$k} = $v;
	    }
	    
	    $buildwriter->startTag('fieldList');
	    foreach $f (keys %$fieldmap) {
		$field = $fieldmap->{$f};
		$buildwriter->emptyTag('field', 'shortname'=>$field, 'name'=>$f);
	    }	
	    $buildwriter->endTag('fieldList');
	} else {
	    print STDERR "indexfieldmap not defined";
	}
	
    } elsif ($buildtype eq 'mg') {
	$buildwriter->startTag('serviceRack', 'name'=>'GS2MGSearch');
    }
    
    
    $buildwriter->emptyTag('defaultIndex', 'name'=>$defaultindex);
    $buildwriter->startTag('indexList');
    #for each index
    foreach $i (keys %$indexmap) {
	$index = $indexmap->{$i};
	$buildwriter->emptyTag('index', 'name'=>$index);
    }	
    $buildwriter->endTag('indexList');
    
    $buildwriter->endTag('serviceRack');
   
    $buildwriter->endTag('serviceRackList');
    $buildwriter->endTag('buildConfig');
    $collwriter->endTag('collectionConfig');
    $buildwriter->end();
    $buildoutput->close();
    $colloutput->close();
}


sub outputmetadata {
    my ($writer, $metaname, $lang, $metavalue) = @_;
    $lang = 'en' if $lang eq 'default';
    if ($lang =~ /^\[/) {
	($lang) = $lang =~ /\[l=(.*)\]/;
    }
    $writer->startTag('metadata',  'name'=>$metaname, 'lang'=>$lang);
    $writer->characters($metavalue);
    $writer->endTag('metadata');
}

sub output_display {
    my ($writer, $lang, $value) = @_;
    $lang = 'en' if $lang eq 'default';
    if ($lang =~ /^\[/) {
	($lang) = $lang =~ /\[l=(.*)\]/;
    }
    $writer->startTag('displayName', 'lang'=>$lang);
    $writer->characters($value);
    $writer->endTag('displayName');
}
sub format_icon_value {
    my ($value) = @_;
    if ($value =~ /^_/) {
	($newvalue) = $value =~ /\/([^\/]*)$/;
	if ($newvalue) {
	    return $newvalue;
	}
    }
    return $value;
}

sub write_format {
    my ($writer, $old_format) = @_;
    #convert [] to <gsf:...>
    #remove IFs, for now just make the first option true
    $old_format =~ s/\{If\}\{[^,]*,([^,\}]*)(,[^\}]*)?\}/$1/g;
    #remove ORs, for now just make the first option true
    $old_format =~ s/\{Or\}\{([^,]*),[^\}]*\}/$1/g;
    $old_format =~ s/\[Text\]/\<gsf:text\/\>/g;
    $old_format =~ s/\[num\]/\<gsf:num\/\>/g;
    $old_format =~ s/\[link\]/\<gsf:link\>/g;
    $old_format =~ s/\[\/link\]/\<\/gsf:link\>/g;
    $old_format =~ s/\[srclink\]/\<gsf:link type=\'source\'\>/g;
    $old_format =~ s/\[\/srclink\]/\<\/gsf:link\>/g;
    $old_format =~ s/\[icon\]/\<gsf:icon\/\>/g;
    $old_format =~ s/\[srcicon\]/\<gsf:icon type=\'source\'\/\>/g;

    #now do the rest of the {} which are assumed to be metadata
    $old_format =~ s/\[([^\]]*\:)?([^\]\:]*)\]/\<gsf:metadata name=\'$2\' select=\'$1\'\/\>/g;
    #do the parent stuff
    $old_format =~ s/(select=\'parent)\:\'/$1\'/g;
    $old_format =~ s/select=\'parent\(Top\)\:\'/select=\'root\'/g;
    $old_format =~ s/select=\'parent\(All\)\:\'/select=\'ancestors\'/g;
    $old_format =~ s/select=\'parent\(All\'([^\']*)\'\)\:\'/select=\'ancestors\' separator=\'$1\'/g;
    #remove any select=''
    $old_format =~ s/select=\'\'//g;
    #turn <br> into <br />
    $old_format =~ s/\<br\>/\<br \/\>/g;
    #turn <p> into <p />
    $old_format =~ s/\<p\>/\<p \/\>/g;
    
    #put quotes around any atts
    $old_format =~ s/=([a-z]+)([> ])/=\'$1\'$2/g;

    $writer->startTag('gsf:template', 'match'=>'documentNode');
    $writer->charactersXML($old_format);
    $writer->endTag('gsf:template');

    
}


sub write_nodelinks_format {
    
    my ($writer) = @_;
    
    $writer->startTag('gsf:template', 'match'=>'classifierNode');
    $writer->startTag('td');
    $writer->startTag('gsf:link', 'type'=>'classifier');
    $writer->emptyTag('gsf:metadata', 'name'=>'Title');
    $writer->endTag('gsf:link');
    $writer->endTag('td');
    $writer->endTag('gsf:template');
    
}
#$writer->startTag('');
#$writer->endTag('');
#$writer->characters();
#$writer->emptyTag('');

1;
